#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This script reads GeoTIFF files each of which is for one spectral 
# band of a Sentinel-1/2 image patch pair in the BigEarthNet v1.0 archive.
# 
# The script is capable of reading either  all spectral bands of one patch 
# folder (-p option) or all bands for all patches (-r option).
# 
# After reading files, Sentinel-1/2 image patch pair values can be used as numpy array 
# for further purposes.
# 
# read_patch --help can be used to learn how to use this script.
#
# Date: 22 Dec 2020
# Version: 1.0.1
# Usage: read_patch.py [-h] [-p PATCH_FOLDER] [-r ROOT_FOLDER]

from __future__ import print_function
import argparse
import os
import json

parser = argparse.ArgumentParser(
    description='This script reads the BigEarthNet v1.0 image patch pairs')
parser.add_argument('-p1', '--patch_folder_s1',
                    dest='patch_folder', help='patch folder path')
parser.add_argument('-r1', '--root_folder_s1', dest='root_folder_s1',
                    help='root folder path contains multiple patch folders of BigEarthNet-S1 v1.0')
parser.add_argument('-r2', '--root_folder_s2', dest='root_folder_s2',
                    help='root folder path contains multiple patch folders of BigEarthNet-S2 v1.0')

args = parser.parse_args()

# Checks the existence of patch folders and populate the list of patch folder paths
folder_path_list = []
if args.root_folder_s1 and args.root_folder_s2:
    print('INFO: -p argument will not be considered since -r argument is defined')
    if not os.path.exists(args.root_folder_s1):
        print('ERROR: folder', args.root_folder_s1, 'does not exist')
        exit()
    elif not os.path.exists(args.root_folder_s2):
        print('ERROR: folder', args.root_folder_s2, 'does not exist')
        exit()
    else:
        folder_path_list = [i[0] for i in os.walk(args.root_folder_s1)][1:-1]
        if len(folder_path_list) == 0:
            print('ERROR: there is no patch directories in the root folder')
            exit()
elif not args.patch_folder_s1:
    print('ERROR: at least one of -p and -r arguments is required')
    exit()
else:
    if not os.path.exists(args.patch_folder_s1):
        print('ERROR: folder', args.patch_folder, 'does not exist')
        exit()
    else:
        folder_path_list = [args.patch_folder_s1]

# Checks the existence of required python packages
gdal_existed = rasterio_existed = georasters_existed = False
try:
    import gdal
    gdal_existed = True
    print('INFO: GDAL package will be used to read GeoTIFF files')
except ImportError:
    try:
        import rasterio
        rasterio_existed = True
        print('INFO: rasterio package will be used to read GeoTIFF files')
    except ImportError:
        print('ERROR: please install either GDAL or rasterio package to read GeoTIFF files')
        exit()

# radar and spectral band names to read related GeoTIFF files
band_names_s1 = ['VV', 'VH']
band_names_s2 = ['B01', 'B02', 'B03', 'B04', 'B05', 'B06', 
                 'B07', 'B08', 'B8A', 'B09', 'B11', 'B12']

def read_bands(patch_folder_path, patch_name, band_names):
    for band_name in band_names:
        # First finds related GeoTIFF path and reads values as an array
        band_path = os.path.join(
            patch_folder_path, patch_name + '_' + band_name + '.tif')
        if gdal_existed:
            band_ds = gdal.Open(band_path,  gdal.GA_ReadOnly)
            raster_band = band_ds.GetRasterBand(1)
            band_data = raster_band.ReadAsArray()
        elif rasterio_existed:
            band_ds = rasterio.open(band_path)
            band_data = band_ds.read(1)
        # band_data keeps the values of band band_name for the patch patch_name
        print('INFO: band', band_name, 'of patch', patch_name,
              'is ready with size', band_data.shape)

# Reads spectral bands of all patches whose folder names are populated before
for patch_folder_path in folder_path_list:
    patch_folder_path = os.path.realpath(patch_folder_path)
    patch_name = os.path.basename(patch_folder_path)

    # read S1 bands
    read_bands(patch_folder_path, patch_name, band_names_s1)

    labels_metadata_path = os.path.join(patch_folder_path, 
                            patch_name + '_labels_metadata.json')
    
    # Reads labels_metadata json file 
    with open(labels_metadata_path, 'r') as f:
        labels_metadata = json.load(f)

    # get corresponding s2 patch
    patch_name = labels_metadata["corresponding_s2_patch"]
    
    # read corresponding S2 bands
    patch_folder_path = os.path.join(args.root_folder_s2, patch_name)
    read_bands(patch_folder_path, patch_name, band_names_s2)
